/*
 * Copyright (c) 2003 The FreeBSD Project. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  Questions/Comments: Mike Makonnen <mtm@FreeBSD.org>
 *
 * $FreeBSD$
 */

#include <cstring>
#include <assert.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/errno.h>
#include <testharness/harness.h>

using namespace std;

static int maxfd;

void
init()
{
	maxfd = getdtablesize();
}

void
useup_desc(int *ia)
{
	int i;
	bzero(ia, sizeof ia);
	for (i = 1; i < maxfd; i++)
		if (fcntl(i, F_GETFL) == -1)
			ia[i] = dup(0);
}

void
closeup_desc(int *ia)
{
	int i;
	for (i = 1; i < maxfd; i++)
		if (ia[i] != 0)
			close(ia[i]);
}

TEST(dup, LowestDesc)
{
	/* Get lowest available file desc. */
	int newd, maxfd, i;
	maxfd = getdtablesize();
	for (i = 0; i <= maxfd && fcntl(i, F_GETFL) != -1; i++)
		;
	assert(i <= maxfd + 1);
	newd = dup(0);
	CHECK_LONGS_EQUAL(i, newd);
}

TEST(dup, errno1)
{
	/* Invalid fd */
	int newd;
	newd = dup(maxfd + 1);
	CHECK_ERRNO(newd, EBADF);
	newd = dup(-1);
	CHECK_ERRNO(newd, EBADF);
}

TEST(dup, errno2)
{
	/* no open descriptors */
	int newd, intarr[maxfd];
	useup_desc(intarr);
	newd = dup(0);
	CHECK_ERRNO(newd, EMFILE);
	closeup_desc(intarr);
}

TEST(dup2, errno1)
{
	/* invalid fd */
	int newd;
	newd = dup2(0, maxfd + 1);
	CHECK_ERRNO(newd, EBADF);
	newd = dup2(0, -1);
	CHECK_ERRNO(newd, EBADF);
}

int main()
{
	init();
	TestResult tr;
	TestRegistry::runAllTests(tr);
	return (0);
}
