/*
 * Copyright (c) 2003 The FreeBSD Project. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  Questions/Comments: Mike Makonnen <mtm@FreeBSD.org>
 *
 * $FreeBSD$
 */

#include <cstring>
#include <fcntl.h>
#include <unistd.h>
#include <testharness/harness.h>

using namespace std;

static int maxfd;

void
init()
{
	assert(fcntl(0, F_GETFL) != -1);
	maxfd = getdtablesize();
}

void
useup_fd(int *pa)
{
	int i;
	/* XXX - assume it's not dynamically allocated */
	bzero(pa, sizeof pa);
	for (i = 1; i < maxfd; i++)
		if (fcntl(i, F_GETFL) == -1)
			pa[i] = dup(0);
}

void
closeup_fd(int *pa, int num)
{
	int i;
	num = num == 0 ? maxfd : num;
	for (i = 1; i < num; i++)
		if (pa[i] != 0) {
			close(pa[i]);
			pa[i] = 0;
		}
}

TEST(fcntl, errno1)
{
	/* errno == EMFILE: No file descs. greater/equal arg are available */
	int newd, i, ia[maxfd];
	useup_fd(ia);
	closeup_fd(ia, maxfd/10);
	assert(maxfd/10 + 3 < maxfd - 1);
	newd = fcntl(0, F_DUPFD, maxfd/10 + 3 );
	CHECK_ERRNO(newd, EMFILE);
	closeup_fd(ia, 0);
}

TEST(fcntl, errno2)
{
	/* errno == EMFILE: max. # of file desc. already open */
	int newd, i, intarr[maxfd];
	useup_fd(intarr);
	newd = fcntl(0, F_DUPFD, 1);
	CHECK_ERRNO(newd, EMFILE);
	closeup_fd(intarr, 0);
}

TEST(fcntl, errno3)
{
	/* EINVAL if arg > maxfilesperproc */
	int newd;
	newd = fcntl(0, F_DUPFD, maxfd + 1);
	CHECK_ERRNO(newd, EINVAL);
}

TEST(fcntl, errno4)
{
	/* EINVAL if arg < 0 */
	int newd;
	newd = fcntl(0, F_DUPFD, -1);
	CHECK_ERRNO(newd, EINVAL);
}

int main()
{
	TestResult tr;
	init();
	TestRegistry::runAllTests(tr);
	return (0);
}
